# version 430

precision mediump float;

//material settings
uniform sampler2D color_texture; 
uniform sampler2D normal_texture;
uniform sampler2D light_texture;
uniform float shininess; 
uniform vec3 specularMatColor; 
uniform float gamma;
uniform int hasNormalMap;
uniform int hasLightMap;
uniform mat4 normalMat;
uniform mat3 V3x3;
uniform mat3 M3x3;
uniform int showNormals;

uniform struct Light{
	vec3 position;
	vec3 intensities;
	float ambientCoefficient;
	float coneAngle;
	vec3 coneDirection;
} lightsources[4];

in vec3 worldNormal;
in vec3 worldPos; 
in vec2 texCoord;
in vec3 _normal;
in vec3 _tangent;
in vec3 _bitangent;


layout(location=0) out vec4 outColor;


vec3 applyLight(Light light, vec3 surfaceColor, vec3 normal, vec3 surfacePos){
	
	vec3 surfaceToLight = normalize(light.position-surfacePos);

	//ambient
	vec3 ambient = light.ambientCoefficient * surfaceColor.rgb * light.intensities;

	//diffuse
	float diffuseCoefficient = max(0.0, dot(normal, surfaceToLight));
	vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * light.intensities;
	
	//specular
	float specularCoefficient = 0.0;
	if(diffuseCoefficient > 0.0) {
		vec3 viewDir = normalize(-surfacePos);
		vec3 halfDir = normalize(surfaceToLight + viewDir);
		float specAngle = max(dot(halfDir, normal), 0.0);
		float spec = pow(specAngle, shininess);
	}
	vec3 specular = specularCoefficient * specularMatColor * light.intensities;
	

	//spotlight regulations
	vec3 coneDirection = normalize(light.coneDirection);
	vec3 rayDirection = -surfaceToLight;
	float lightToSurfaceAngle = degrees(acos(dot(rayDirection, coneDirection)));
	float factor = smoothstep(light.coneAngle, 50, lightToSurfaceAngle);

	return ambient + factor*(diffuse + specular);
}

void main() {
	
	vec3 N;
	if (hasNormalMap == 1)
	{		
		mat3 _tbn = mat3(
   			_tangent,
    		_bitangent,
    		_normal
		);

		vec3 normal2;
		normal2 = normalize(texture(normal_texture, texCoord).rgb * 2.0 - 1.0);
		normal2 = normalize(mat3(normalMat) * _tbn * normal2); 

		N = normalize(normal2);
	}
	else
	{
		N = normalize(worldNormal);
	}

	vec4 surfaceColor = texture(color_texture, texCoord);
	
	vec3 linearColor = vec3(0);
	for(int i = 0; i < 4; i++){
		linearColor += applyLight(lightsources[i], surfaceColor.xyz, N, worldPos);
	}


	vec3 gamma = vec3(1.0/gamma);
	if (showNormals == 1)
	{
	outColor = vec4(N, 1.0);
	}
	else
	{
	outColor = vec4(pow(linearColor.xyz, gamma), surfaceColor.a);
	}

}