# version 430

precision mediump float;

//material settings
uniform sampler2D color_texture; 
uniform sampler2D normal_texture;
uniform sampler2D light_texture;
uniform float gamma;
uniform int hasNormalMap;
uniform int hasLightMap;
uniform mat3 M3x3;

in vec3 worldNormal;
in vec3 worldPos; 
in vec2 texCoord; 

in mat3 _tbn;

layout(location=0) out vec4 outColor;

void main() {
	
	vec4 surfaceColor = texture(color_texture, texCoord);
	
	vec3 diffuse = vec3(0);
	vec3 lightMap = texture(light_texture, texCoord).xyz;

	if(((lightMap.x+lightMap.y+lightMap.z)/3) > 0.90){
		lightMap.x = 1.0;
		lightMap.y = 1.0;
		lightMap.z = 1.0;
	} else if (((lightMap.x+lightMap.y+lightMap.z)/3) > 0.70){
		lightMap.x = 0.8;
		lightMap.y = 0.8;
		lightMap.z = 0.8;
	} else if (((lightMap.x+lightMap.y+lightMap.z)/3) > 0.50){
		lightMap.x = 0.6;
		lightMap.y = 0.6;
		lightMap.z = 0.6;
	} else if (((lightMap.x+lightMap.y+lightMap.z)/3) > 0.30){
		lightMap.x = 0.4;
		lightMap.y = 0.4;
		lightMap.z = 0.4;
	} else if (((lightMap.x+lightMap.y+lightMap.z)/3) > 0.10){
		lightMap.x = 0.2;
		lightMap.y = 0.2;
		lightMap.z = 0.2;
	} else {
		lightMap.x = 0.0;
		lightMap.y = 0.0;
		lightMap.z = 0.0;
	}

	vec3 diffuseColor = lightMap * surfaceColor.xyz;

	vec3 ambient = 0.03 * surfaceColor.rgb;

	vec3 linearColor = diffuseColor + ambient;

	vec3 gamma = vec3(1.0/gamma);
	outColor = vec4(pow(linearColor, gamma), surfaceColor.a);
}